@@ -9,7 +9,7 @@ the book "Guide to NumPy" at
 
 It is being distributed for a fee for only a few years to
 cover some of the costs of development.  After the restriction period
-it will also be freely available. 
+it will also be freely available.
 
 Additional documentation is available in the docstrings and at
 
@@ -17,7 +17,7 @@ http://www.scipy.org.
 """
 
 try:
-    from __config__ import show as show_config
+    from numpy.__config__ import show as show_config
 except ImportError:
     show_config = None
 
@@ -29,7 +29,10 @@ else:
     from version import version as __version__
 
     from _import_tools import PackageLoader
-    pkgload = PackageLoader()
+
+    def pkgload(*packages, **options):
+        loader = PackageLoader(infunc=True)
+        return loader(*packages, **options)
 
     import testing
     from testing import ScipyTest, NumpyTest
@@ -53,7 +56,7 @@ else:
     __all__ += core.__all__
     __all__ += lib.__all__
     __all__ += ['linalg', 'fft', 'random', 'ctypeslib']
-        
+
     if __doc__ is not None:
         __doc__ += """
 
@@ -66,26 +69,27 @@ lib       --- Basic functions used by several sub-packages and useful
 random    --- Core Random Tools
 linalg    --- Core Linear Algebra Tools
 fft       --- Core FFT routines
-testing   --- Scipy testing tools
+testing   --- Numpy testing tools
 
   These packages require explicit import
-f2py      --- Fortran to Python Interface Generator. 
-distutils --- Enhancements to distutils with support for 
-              Fortran compilers support and more. 
+f2py      --- Fortran to Python Interface Generator.
+distutils --- Enhancements to distutils with support for
+              Fortran compilers support and more.
 
 
 Global symbols from subpackages
 -------------------------------
 core    --> *
 lib     --> *
-testing --> ScipyTest, NumpyTest
+testing --> NumpyTest
 """
 
-    def test(level=1, verbosity=1):
-        if level <= 10:	
-           return NumpyTest().test(level, verbosity)
-        else:
-           return NumpyTest().testall(level, verbosity)
+    def test(*args, **kw):
+        import os, sys
+        print 'Numpy is installed in %s' % (os.path.split(__file__)[0],)
+        print 'Numpy version %s' % (__version__,)
+        print 'Python version %s' % (sys.version.replace('\n', '',),)
+        return NumpyTest().test(*args, **kw)
     test.__doc__ = NumpyTest.test.__doc__
 
     import add_newdocs
