@@ -1,10 +1,10 @@
 /* #define SIZEOF_SHORT 2 */
 /* #define SIZEOF_INT 4 */
-/* #define SIZEOF_LONG 8 */
+/* #define SIZEOF_LONG 4 */
 /* #define SIZEOF_FLOAT 4 */
 /* #define SIZEOF_DOUBLE 8 */
-#define SIZEOF_LONG_DOUBLE 16
-#define SIZEOF_PY_INTPTR_T 8
+#define SIZEOF_LONG_DOUBLE 12
+#define SIZEOF_PY_INTPTR_T 4
 /* #define SIZEOF_LONG_LONG 8 */
 #define SIZEOF_PY_LONG_LONG 8
 /* #define CHAR_BIT 8 */
@@ -17,3 +17,8 @@
 #define HAVE_ISNAN
 #define HAVE_ISINF
 #define HAVE_RINT
+#ifdef WITH_THREAD
+#define NPY_ALLOW_THREADS 1
+#else
+#define NPY_ALLOW_THREADS 0
+#endif
