@@ -2,7 +2,7 @@ import numpy
 import types, time
 from numpy.core.ma import *
 from numpy.core.numerictypes import float32
-from numpy.testing import ScipyTestCase, ScipyTest
+from numpy.testing import NumpyTestCase, NumpyTest
 pi = numpy.pi
 def eq(v,w, msg=''):
     result = allclose(v,w)
@@ -13,9 +13,9 @@ def eq(v,w, msg=''):
 %s"""% (msg, str(v), str(w))
     return result
 
-class test_ma(ScipyTestCase):
+class TestMa(NumpyTestCase):
     def __init__(self, *args, **kwds):
-        ScipyTestCase.__init__(self, *args, **kwds)
+        NumpyTestCase.__init__(self, *args, **kwds)
         self.setUp()
 
     def setUp (self):
@@ -86,11 +86,15 @@ class test_ma(ScipyTestCase):
             self.failUnless(eq(x + y, xm + ym))
             self.failUnless(eq(x - y, xm - ym))
             self.failUnless(eq(x * y, xm * ym))
+            olderr = numpy.seterr(divide='ignore', invalid='ignore')
             self.failUnless(eq(x / y, xm / ym))
+            numpy.seterr(**olderr)
             self.failUnless(eq(a10 + y, a10 + ym))
             self.failUnless(eq(a10 - y, a10 - ym))
             self.failUnless(eq(a10 * y, a10 * ym))
+            olderr = numpy.seterr(divide='ignore', invalid='ignore')
             self.failUnless(eq(a10 / y, a10 / ym))
+            numpy.seterr(**olderr)
             self.failUnless(eq(x + a10, xm + a10))
             self.failUnless(eq(x - a10, xm - a10))
             self.failUnless(eq(x * a10, xm * a10))
@@ -101,7 +105,9 @@ class test_ma(ScipyTestCase):
             self.failUnless(eq(numpy.add(x,y), add(xm, ym)))
             self.failUnless(eq(numpy.subtract(x,y), subtract(xm, ym)))
             self.failUnless(eq(numpy.multiply(x,y), multiply(xm, ym)))
+            olderr = numpy.seterr(divide='ignore', invalid='ignore')
             self.failUnless(eq(numpy.divide(x,y), divide(xm, ym)))
+            numpy.seterr(**olderr)
 
 
     def check_testMixedArithmetic(self):
@@ -119,9 +125,11 @@ class test_ma(ScipyTestCase):
         self.failUnless (eq(numpy.sinh(x), sinh(xm)))
         self.failUnless (eq(numpy.tan(x), tan(xm)))
         self.failUnless (eq(numpy.tanh(x), tanh(xm)))
+        olderr = numpy.seterr(divide='ignore', invalid='ignore')
         self.failUnless (eq(numpy.sqrt(abs(x)), sqrt(xm)))
         self.failUnless (eq(numpy.log(abs(x)), log(xm)))
         self.failUnless (eq(numpy.log10(abs(x)), log10(xm)))
+        numpy.seterr(**olderr)
         self.failUnless (eq(numpy.exp(x), exp(xm)))
         self.failUnless (eq(numpy.arcsin(z), arcsin(zm)))
         self.failUnless (eq(numpy.arccos(z), arccos(zm)))
@@ -167,14 +175,14 @@ class test_ma(ScipyTestCase):
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
         self.failUnless (eq(numpy.add.reduce(x), add.reduce(x)))
         self.failUnless (eq(numpy.add.accumulate(x), add.accumulate(x)))
-        self.failUnless (eq(4, sum(array(4))))
+        self.failUnless (eq(4, sum(array(4),axis=0)))
         self.failUnless (eq(4, sum(array(4), axis=0)))
-        self.failUnless (eq(numpy.sum(x), sum(x)))
-        self.failUnless (eq(numpy.sum(filled(xm,0)), sum(xm)))
+        self.failUnless (eq(numpy.sum(x,axis=0), sum(x,axis=0)))
+        self.failUnless (eq(numpy.sum(filled(xm,0),axis=0), sum(xm,axis=0)))
         self.failUnless (eq(numpy.sum(x,0), sum(x,0)))
-        self.failUnless (eq(numpy.product(x), product(x)))
+        self.failUnless (eq(numpy.product(x,axis=0), product(x,axis=0)))
         self.failUnless (eq(numpy.product(x,0), product(x,0)))
-        self.failUnless (eq(numpy.product(filled(xm,1)), product(xm)))
+        self.failUnless (eq(numpy.product(filled(xm,1),axis=0), product(xm,axis=0)))
         if len(s) > 1:
             self.failUnless (eq(numpy.concatenate((x,y),1), concatenate((xm,ym),1)))
             self.failUnless (eq(numpy.add.reduce(x,1), add.reduce(x,1)))
@@ -267,9 +275,9 @@ class test_ma(ScipyTestCase):
         y4 = resize(x4, (8,))
         self.failUnless( eq(concatenate([x4,x4]), y4))
         self.failUnless( eq(getmask(y4),[0,0,1,0,0,0,1,0]))
-        y5 = repeat(x4, (2,2,2,2))
+        y5 = repeat(x4, (2,2,2,2), axis=0)
         self.failUnless( eq(y5, [0,0,1,1,2,2,3,3]))
-        y6 = repeat(x4, 2)
+        y6 = repeat(x4, 2, axis=0)
         self.failUnless( eq(y5, y6))
 
     def check_testPut(self):
@@ -306,11 +314,11 @@ class test_ma(ScipyTestCase):
     def check_testMaPut(self):
         (x, y, a10, m1, m2, xm, ym, z, zm, xf, s) = self.d
         m = [1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1]
-        i = numpy.nonzero(m)
+        i = numpy.nonzero(m)[0]
         putmask(xm, m, z)
-        assert take(xm, i) == z
+        assert take(xm, i,axis=0) == z
         put(ym, i, zm)
-        assert take(ym, i) == zm
+        assert take(ym, i,axis=0) == zm
 
     def check_testOddFeatures(self):
         "Test of other odd features"
@@ -424,10 +432,10 @@ class test_ma(ScipyTestCase):
         y=y.reshape(2,3,4)
         assert eq(numpy.transpose(y,(2,0,1)), transpose(x,(2,0,1)))
         assert eq(numpy.take(y, (2,0,1), 1), take(x, (2,0,1), 1))
-        assert eq(numpy.innerproduct(filled(x,0),filled(y,0)),
-                                innerproduct(x, y))
-        assert eq(numpy.outerproduct(filled(x,0),filled(y,0)),
-                                outerproduct(x, y))
+        assert eq(numpy.inner(filled(x,0),filled(y,0)),
+                                inner(x, y))
+        assert eq(numpy.outer(filled(x,0),filled(y,0)),
+                                outer(x, y))
         y = array(['abc', 1, 'def', 2, 3], object)
         y[2] = masked
         t = take(y,[0,3,4])
@@ -513,20 +521,20 @@ class test_ma(ScipyTestCase):
     def check_testAverage1(self):
         "Test of average."
         ott = array([0.,1.,2.,3.], mask=[1,0,0,0])
-        self.failUnless(eq(2.0, average(ott)))
+        self.failUnless(eq(2.0, average(ott,axis=0)))
         self.failUnless(eq(2.0, average(ott, weights=[1., 1., 2., 1.])))
         result, wts = average(ott, weights=[1.,1.,2.,1.], returned=1)
         self.failUnless(eq(2.0, result))
         self.failUnless(wts == 4.0)
         ott[:] = masked
-        self.failUnless(average(ott) is masked)
+        self.failUnless(average(ott,axis=0) is masked)
         ott = array([0.,1.,2.,3.], mask=[1,0,0,0])
         ott=ott.reshape(2,2)
         ott[:,1] = masked
-        self.failUnless(eq(average(ott), [2.0, 0.0]))
+        self.failUnless(eq(average(ott,axis=0), [2.0, 0.0]))
         self.failUnless(average(ott,axis=1)[0] is masked)
-        self.failUnless(eq([2.,0.], average(ott)))
-        result, wts = average(ott, returned=1)
+        self.failUnless(eq([2.,0.], average(ott, axis=0)))
+        result, wts = average(ott, axis=0, returned=1)
         self.failUnless(eq(wts, [1., 0.]))
 
     def check_testAverage2(self):
@@ -534,30 +542,30 @@ class test_ma(ScipyTestCase):
         w1 = [0,1,1,1,1,0]
         w2 = [[0,1,1,1,1,0],[1,0,0,0,0,1]]
         x=arange(6)
-        self.failUnless(allclose(average(x), 2.5))
-        self.failUnless(allclose(average(x, weights=w1), 2.5))
+        self.failUnless(allclose(average(x, axis=0), 2.5))
+        self.failUnless(allclose(average(x, axis=0, weights=w1), 2.5))
         y=array([arange(6), 2.0*arange(6)])
         self.failUnless(allclose(average(y, None), numpy.add.reduce(numpy.arange(6))*3./12.))
         self.failUnless(allclose(average(y, axis=0), numpy.arange(6) * 3./2.))
-        self.failUnless(allclose(average(y, axis=1), [average(x), average(x) * 2.0]))
+        self.failUnless(allclose(average(y, axis=1), [average(x,axis=0), average(x,axis=0) * 2.0]))
         self.failUnless(allclose(average(y, None, weights=w2), 20./6.))
         self.failUnless(allclose(average(y, axis=0, weights=w2), [0.,1.,2.,3.,4.,10.]))
-        self.failUnless(allclose(average(y, axis=1), [average(x), average(x) * 2.0]))
+        self.failUnless(allclose(average(y, axis=1), [average(x,axis=0), average(x,axis=0) * 2.0]))
         m1 = zeros(6)
         m2 = [0,0,1,1,0,0]
         m3 = [[0,0,1,1,0,0],[0,1,1,1,1,0]]
         m4 = ones(6)
         m5 = [0, 1, 1, 1, 1, 1]
-        self.failUnless(allclose(average(masked_array(x, m1)), 2.5))
-        self.failUnless(allclose(average(masked_array(x, m2)), 2.5))
-        self.failUnless(average(masked_array(x, m4)) is masked)
-        self.assertEqual(average(masked_array(x, m5)), 0.0)
-        self.assertEqual(count(average(masked_array(x, m4))), 0)
+        self.failUnless(allclose(average(masked_array(x, m1),axis=0), 2.5))
+        self.failUnless(allclose(average(masked_array(x, m2),axis=0), 2.5))
+        self.failUnless(average(masked_array(x, m4),axis=0) is masked)
+        self.assertEqual(average(masked_array(x, m5),axis=0), 0.0)
+        self.assertEqual(count(average(masked_array(x, m4),axis=0)), 0)
         z = masked_array(y, m3)
         self.failUnless(allclose(average(z, None), 20./6.))
         self.failUnless(allclose(average(z, axis=0), [0.,1.,99.,99.,4.0, 7.5]))
         self.failUnless(allclose(average(z, axis=1), [2.5, 5.0]))
-        self.failUnless(allclose( average(z,weights=w2), [0.,1., 99., 99., 4.0, 10.0]))
+        self.failUnless(allclose( average(z,axis=0, weights=w2), [0.,1., 99., 99., 4.0, 10.0]))
 
         a = arange(6)
         b = arange(6) * 3
@@ -572,9 +580,9 @@ class test_ma(ScipyTestCase):
         self.failUnless(shape(w2) == shape(r2))
         a2d = array([[1,2],[0,4]], float)
         a2dm = masked_array(a2d, [[0,0],[1,0]])
-        a2da = average(a2d)
+        a2da = average(a2d, axis=0)
         self.failUnless(eq (a2da, [0.5, 3.0]))
-        a2dma = average(a2dm)
+        a2dma = average(a2dm, axis=0)
         self.failUnless(eq( a2dma, [1.0, 3.0]))
         a2dma = average(a2dm, axis=None)
         self.failUnless(eq(a2dma, 7./3.))
@@ -603,7 +611,7 @@ class test_ma(ScipyTestCase):
         self.failUnless(minimum(xm, xm).mask)
         self.failUnless(xm.filled().dtype is xm.data.dtype)
         x = array(0, mask=0)
-        self.failUnless(x.filled() is x.data)
+        self.failUnless(x.filled() == x.data)
         self.failUnlessEqual(str(xm), str(masked_print_option))
 
     def check_testArrayMethods(self):
@@ -639,7 +647,7 @@ class test_ma(ScipyTestCase):
         self.failUnlessEqual(b[0].shape, ())
         self.failUnlessEqual(b[1].shape, ())
 
-class test_ufuncs(ScipyTestCase):
+class TestUfuncs(NumpyTestCase):
     def setUp(self):
         self.d = (array([1.0, 0, -1, pi/2]*2, mask=[0,1]+[0]*6),
                   array([1.0, 0, -1, pi/2]*2, mask=[1,0]+[0]*6),)
@@ -668,20 +676,27 @@ class test_ufuncs(ScipyTestCase):
             try:
                 uf = getattr(umath, f)
             except AttributeError:
-                uf = getattr(oldnumeric, f)
+                uf = getattr(fromnumeric, f)
             mf = getattr(numpy.ma, f)
             args = self.d[:uf.nin]
+            olderr = numpy.geterr()
+            if f in ['sqrt', 'arctanh', 'arcsin', 'arccos', 'arccosh', 'arctanh', 'log',
+                     'log10','divide','true_divide', 'floor_divide', 'remainder', 'fmod']:
+                numpy.seterr(invalid='ignore')
+            if f in ['arctanh', 'log', 'log10']:
+                numpy.seterr(divide='ignore')
             ur = uf(*args)
             mr = mf(*args)
+            numpy.seterr(**olderr)
             self.failUnless(eq(ur.filled(0), mr.filled(0), f))
             self.failUnless(eqmask(ur.mask, mr.mask))
 
     def test_reduce(self):
         a = self.d[0]
-        self.failIf(alltrue(a))
-        self.failUnless(sometrue(a))
-        self.failUnlessEqual(sum(a[:3]), 0)
-        self.failUnlessEqual(product(a), 0)
+        self.failIf(alltrue(a,axis=0))
+        self.failUnless(sometrue(a,axis=0))
+        self.failUnlessEqual(sum(a[:3],axis=0), 0)
+        self.failUnlessEqual(product(a,axis=0), 0)
 
     def test_minmax(self):
         a = arange(1,13).reshape(3,4)
@@ -699,33 +714,33 @@ class test_ufuncs(ScipyTestCase):
             self.failUnless(eq(nonzero(x), [0]))
 
 
-class test_array_methods(ScipyTestCase):
-        
+class TestArrayMethods(NumpyTestCase):
+
     def setUp(self):
-        x = numpy.array([ 8.375,  7.545,  8.828,  8.5  ,  1.757,  5.928,  
-                          8.43 ,  7.78 ,  9.865,  5.878,  8.979,  4.732,  
-                          3.012,  6.022,  5.095,  3.116,  5.238,  3.957,  
+        x = numpy.array([ 8.375,  7.545,  8.828,  8.5  ,  1.757,  5.928,
+                          8.43 ,  7.78 ,  9.865,  5.878,  8.979,  4.732,
+                          3.012,  6.022,  5.095,  3.116,  5.238,  3.957,
                           6.04 ,  9.63 ,  7.712,  3.382,  4.489,  6.479,
-                          7.189,  9.645,  5.395,  4.961,  9.894,  2.893,  
+                          7.189,  9.645,  5.395,  4.961,  9.894,  2.893,
                           7.357,  9.828,  6.272,  3.758,  6.693,  0.993])
         X = x.reshape(6,6)
         XX = x.reshape(3,2,2,3)
-    
-        m = numpy.array([0, 1, 0, 1, 0, 0, 
-                         1, 0, 1, 1, 0, 1, 
-                         0, 0, 0, 1, 0, 1, 
-                         0, 0, 0, 1, 1, 1, 
-                         1, 0, 0, 1, 0, 0, 
+
+        m = numpy.array([0, 1, 0, 1, 0, 0,
+                         1, 0, 1, 1, 0, 1,
+                         0, 0, 0, 1, 0, 1,
+                         0, 0, 0, 1, 1, 1,
+                         1, 0, 0, 1, 0, 0,
                          0, 0, 1, 0, 1, 0])
         mx = array(data=x,mask=m)
         mX = array(data=X,mask=m.reshape(X.shape))
         mXX = array(data=XX,mask=m.reshape(XX.shape))
-    
-        m2 = numpy.array([1, 1, 0, 1, 0, 0, 
-                          1, 1, 1, 1, 0, 1, 
-                          0, 0, 1, 1, 0, 1, 
-                          0, 0, 0, 1, 1, 1, 
-                          1, 0, 0, 1, 1, 0, 
+
+        m2 = numpy.array([1, 1, 0, 1, 0, 0,
+                          1, 1, 1, 1, 0, 1,
+                          0, 0, 1, 1, 0, 1,
+                          0, 0, 0, 1, 1, 1,
+                          1, 0, 0, 1, 1, 0,
                           0, 0, 1, 0, 1, 1])
         m2x = array(data=x,mask=m2)
         m2X = array(data=X,mask=m2.reshape(X.shape))
@@ -738,7 +753,7 @@ class test_array_methods(ScipyTestCase):
         mXdiag = mX.diagonal()
         self.assertEqual(mX.trace(), mX.diagonal().compressed().sum())
         self.failUnless(eq(mX.trace(),
-                           X.trace() - sum(mXdiag.mask*X.diagonal())))
+                           X.trace() - sum(mXdiag.mask*X.diagonal(),axis=0)))
 
     def test_clip(self):
         (x,X,XX,m,mx,mX,mXX,) = self.d
@@ -774,14 +789,14 @@ class test_array_methods(ScipyTestCase):
         self.failUnless(eq(mXcp.data,mX.filled(1).cumprod(0)))
         mXcp = mX.cumprod(1)
         self.failUnless(eq(mXcp.data,mX.filled(1).cumprod(1)))
-                           
+
     def test_cumsum(self):
         (x,X,XX,m,mx,mX,mXX,) = self.d
         mXcp = mX.cumsum(0)
         self.failUnless(eq(mXcp.data,mX.filled(0).cumsum(0)))
         mXcp = mX.cumsum(1)
         self.failUnless(eq(mXcp.data,mX.filled(0).cumsum(1)))
-                           
+
     def test_varstd(self):
         (x,X,XX,m,mx,mX,mXX,) = self.d
         self.failUnless(eq(mX.var(axis=None),mX.compressed().var()))
@@ -794,7 +809,7 @@ class test_array_methods(ScipyTestCase):
             self.failUnless(eq(mXvar0[k],mX[:,k].compressed().var()))
             self.failUnless(eq(numpy.sqrt(mXvar0[k]),
                                mX[:,k].compressed().std()))
-        
+
 
 def eqmask(m1, m2):
     if m1 is nomask:
@@ -854,5 +869,5 @@ def testinplace(x):
 testinplace.test_name = 'Inplace operations'
 
 if __name__ == "__main__":
-    ScipyTest('numpy.core.ma').run()
+    NumpyTest('numpy.core.ma').run()
     #timingTest()
