@@ -1,12 +1,12 @@
 __all__ = ['atleast_1d','atleast_2d','atleast_3d','vstack','hstack',
-           'column_stack','dstack','array_split','split','hsplit',
+           'column_stack','row_stack', 'dstack','array_split','split','hsplit',
            'vsplit','dsplit','apply_over_axes','expand_dims',
-           'apply_along_axis', 'repmat', 'kron']
+           'apply_along_axis', 'kron', 'tile', 'get_array_wrap']
 
 import numpy.core.numeric as _nx
-from numpy.core.numeric import asarray, zeros, newaxis, outerproduct, \
+from numpy.core.numeric import asarray, zeros, newaxis, outer, \
      concatenate, isscalar, array, asanyarray
-from numpy.core.oldnumeric import product, reshape
+from numpy.core.fromnumeric import product, reshape
 
 def apply_along_axis(func1d,axis,arr,*args):
     """ Execute func1d(arr[i],*args) where func1d takes 1-D arrays
@@ -26,12 +26,12 @@ def apply_along_axis(func1d,axis,arr,*args):
     indlist.remove(axis)
     i[axis] = slice(None,None)
     outshape = asarray(arr.shape).take(indlist)
-    i.put(ind, indlist)
+    i.put(indlist, ind)
     res = func1d(arr[tuple(i.tolist())],*args)
     #  if res is a number, then we have a smaller output array
     if isscalar(res):
         outarr = zeros(outshape,asarray(res).dtype)
-        outarr[ind] = res
+        outarr[tuple(ind)] = res
         Ntot = product(outshape)
         k = 1
         while k < Ntot:
@@ -42,9 +42,9 @@ def apply_along_axis(func1d,axis,arr,*args):
                 ind[n-1] += 1
                 ind[n] = 0
                 n -= 1
-            i.put(ind,indlist)
+            i.put(indlist,ind)
             res = func1d(arr[tuple(i.tolist())],*args)
-            outarr[ind] = res
+            outarr[tuple(ind)] = res
             k += 1
         return outarr
     else:
@@ -63,7 +63,7 @@ def apply_along_axis(func1d,axis,arr,*args):
                 ind[n-1] += 1
                 ind[n] = 0
                 n -= 1
-            i.put(ind, indlist)
+            i.put(indlist, ind)
             res = func1d(arr[tuple(i.tolist())],*args)
             outarr[tuple(i.tolist())] = res
             k += 1
@@ -121,7 +121,7 @@ def atleast_1d(*arys):
     """
     res = []
     for ary in arys:
-        res.append(array(ary,copy=False,ndmin=1))
+        res.append(array(ary,copy=False,subok=True,ndmin=1))
     if len(res) == 1:
         return res[0]
     else:
@@ -141,7 +141,7 @@ def atleast_2d(*arys):
     """
     res = []
     for ary in arys:
-        res.append(array(ary,copy=False,ndmin=2))
+        res.append(array(ary,copy=False,subok=True,ndmin=2))
     if len(res) == 1:
         return res[0]
     else:
@@ -184,7 +184,7 @@ def vstack(tup):
     """ Stack arrays in sequence vertically (row wise)
 
         Description:
-            Take a sequence of arrays and stack them veritcally
+            Take a sequence of arrays and stack them vertically
             to make a single array.  All arrays in the sequence
             must have the same shape along all but the first axis.
             vstack will rebuild arrays divided by vsplit.
@@ -238,16 +238,21 @@ def hstack(tup):
     """
     return _nx.concatenate(map(atleast_1d,tup),1)
 
+row_stack = vstack
+
 def column_stack(tup):
     """ Stack 1D arrays as columns into a 2D array
 
         Description:
             Take a sequence of 1D arrays and stack them as columns
             to make a single 2D array.  All arrays in the sequence
-            must have the same length.
+            must have the same first dimension.  2D arrays are
+            stacked as-is, just like with hstack.  1D arrays are turned
+            into 2D columns first.
+
         Arguments:
-            tup -- sequence of 1D arrays.  All arrays must have the same
-                   length.
+            tup -- sequence of 1D or 2D arrays.  All arrays must have the same
+                   first dimension.
         Examples:
             >>> import numpy
             >>> a = array((1,2,3))
@@ -258,35 +263,43 @@ def column_stack(tup):
                    [3, 4]])
 
     """
-    arrays = map(_nx.transpose,map(atleast_2d,tup))
+    arrays = []
+    for v in tup:
+        arr = array(v,copy=False,subok=True)
+        if arr.ndim < 2:
+            arr = array(arr,copy=False,subok=True,ndmin=2).T
+        arrays.append(arr)
     return _nx.concatenate(arrays,1)
 
 def dstack(tup):
     """ Stack arrays in sequence depth wise (along third dimension)
 
-        Description:
-            Take a sequence of arrays and stack them along the third axis.
-            All arrays in the sequence must have the same shape along all
-            but the third axis.  This is a simple way to stack 2D arrays
-            (images) into a single 3D array for processing.
-            dstack will rebuild arrays divided by dsplit.
-        Arguments:
-            tup -- sequence of arrays.  All arrays must have the same
-                   shape.
-        Examples:
-            >>> import numpy
-            >>> a = array((1,2,3))
-            >>> b = array((2,3,4))
-            >>> numpy.dstack((a,b))
-            array([       [[1, 2],
-                    [2, 3],
-                    [3, 4]]])
-            >>> a = array([[1],[2],[3]])
-            >>> b = array([[2],[3],[4]])
-            >>> numpy.dstack((a,b))
-            array([[        [1, 2]],
-                   [        [2, 3]],
-                   [        [3, 4]]])
+    Description:
+        Take a sequence of arrays and stack them along the third axis.
+        All arrays in the sequence must have the same shape along all
+        but the third axis.  This is a simple way to stack 2D arrays
+        (images) into a single 3D array for processing.
+        dstack will rebuild arrays divided by dsplit.
+    Arguments:
+        tup -- sequence of arrays.  All arrays must have the same
+               shape.
+    Examples:
+        >>> import numpy
+        >>> a = array((1,2,3))
+        >>> b = array((2,3,4))
+        >>> numpy.dstack((a,b))
+        array([[[1, 2],
+                [2, 3],
+                [3, 4]]])
+        >>> a = array([[1],[2],[3]])
+        >>> b = array([[2],[3],[4]])
+        >>> numpy.dstack((a,b))
+        array([[[1, 2]],
+        <BLANKLINE>
+               [[2, 3]],
+        <BLANKLINE>
+               [[3, 4]]])
+
     """
     return _nx.concatenate(map(atleast_3d,tup),2)
 
@@ -324,12 +337,7 @@ def array_split(ary,indices_or_sections,axis = 0):
         Caveats:
            Currently, the default for axis is 0.  This
            means a 2D array is divided into multiple groups
-           of rows.  This seems like the appropriate default, but
-           we've agreed most other functions should default to
-           axis=-1.  Perhaps we should use axis=-1 for consistency.
-           However, we could also make the argument that NumPy
-           works on "rows" by default.  sum() sums up rows of
-           values.  split() will split data into rows.  Opinions?
+           of rows.  This seems like the appropriate default,
     """
     try:
         Ntotal = ary.shape[axis]
@@ -389,12 +397,7 @@ def split(ary,indices_or_sections,axis=0):
         Caveats:
            Currently, the default for axis is 0.  This
            means a 2D array is divided into multiple groups
-           of rows.  This seems like the appropriate default, but
-           we've agreed most other functions should default to
-           axis=-1.  Perhaps we should use axis=-1 for consistency.
-           However, we could also make the argument that NumPy
-           works on "rows" by default.  sum() sums up rows of
-           values.  split() will split data into rows.  Opinions?
+           of rows.  This seems like the appropriate default
     """
     try: len(indices_or_sections)
     except TypeError:
@@ -435,6 +438,7 @@ def hsplit(ary,indices_or_sections):
             >>> numpy.hsplit(a,2)
             [array([1, 2]), array([3, 4])]
             >>> a = array([[1,2,3,4],[1,2,3,4]])
+            >>> hsplit(a,2)
             [array([[1, 2],
                    [1, 2]]), array([[3, 4],
                    [3, 4]])]
@@ -481,8 +485,8 @@ def vsplit(ary,indices_or_sections):
             import numpy
             >>> a = array([[1,2,3,4],
             ...            [1,2,3,4]])
-            >>> numpy.vsplit(a)
-            [array([       [1, 2, 3, 4]]), array([       [1, 2, 3, 4]])]
+            >>> numpy.vsplit(a,2)
+            [array([[1, 2, 3, 4]]), array([[1, 2, 3, 4]])]
 
     """
     if len(_nx.shape(ary)) < 2:
@@ -517,8 +521,9 @@ def dsplit(ary,indices_or_sections):
             dstack, split, array_split, hsplit, vsplit.
         Examples:
             >>> a = array([[[1,2,3,4],[1,2,3,4]]])
-            [array([       [[1, 2],
-                    [1, 2]]]), array([       [[3, 4],
+            >>> dsplit(a,2)
+            [array([[[1, 2],
+                    [1, 2]]]), array([[[3, 4],
                     [3, 4]]])]
 
     """
@@ -526,57 +531,103 @@ def dsplit(ary,indices_or_sections):
         raise ValueError, 'vsplit only works on arrays of 3 or more dimensions'
     return split(ary,indices_or_sections,2)
 
-# should figure out how to generalize this one.
-def repmat(a, m, n):
-    """Repeat a 0-d to 2-d array mxn times
-    """
-    a = asarray(a)
-    ndim = a.ndim
-    if ndim == 0:
-        origrows, origcols = (1,1)
-    elif ndim == 1:
-        origrows, origcols = (1, a.shape[0])
-    else:
-        origrows, origcols = a.shape
-    rows = origrows * m
-    cols = origcols * n
-    c = a.reshape(1,a.size).repeat(m, 0).reshape(rows, origcols).repeat(n,0)
-    return c.reshape(rows, cols)
-
-
-def _getwrapper(*args):
+def get_array_wrap(*args):
     """Find the wrapper for the array with the highest priority.
 
     In case of ties, leftmost wins. If no wrapper is found, return None
     """
     wrappers = [(getattr(x, '__array_priority__', 0), -i,
-                 x.__array_wrap__) for i, x in enumerate(args) 
+                 x.__array_wrap__) for i, x in enumerate(args)
                                    if hasattr(x, '__array_wrap__')]
     wrappers.sort()
     if wrappers:
         return wrappers[-1][-1]
-    return None 
+    return None
 
 def kron(a,b):
     """kronecker product of a and b
 
-    Kronecker product of two matrices is block matrix
+    Kronecker product of two arrays is block array
     [[ a[ 0 ,0]*b, a[ 0 ,1]*b, ... , a[ 0 ,n-1]*b  ],
      [ ...                                   ...   ],
      [ a[m-1,0]*b, a[m-1,1]*b, ... , a[m-1,n-1]*b  ]]
     """
-    wrapper = _getwrapper(a, b)
-    a = asanyarray(a)
+    wrapper = get_array_wrap(a, b)
     b = asanyarray(b)
-    if not (len(a.shape) == len(b.shape) == 2):
-        raise ValueError("a and b must both be two dimensional")
+    a = array(a,copy=False,subok=True,ndmin=b.ndim)
+    ndb, nda = b.ndim, a.ndim
+    if (nda == 0 or ndb == 0):
+        return _nx.multiply(a,b)
+    as_ = a.shape
+    bs = b.shape
     if not a.flags.contiguous:
-        a = reshape(a, a.shape)
+        a = reshape(a, as_)
     if not b.flags.contiguous:
-        b = reshape(b, b.shape)
-    o = outerproduct(a,b)
-    o=o.reshape(a.shape + b.shape)
-    result = concatenate(concatenate(o, axis=1), axis=1)
+        b = reshape(b, bs)
+    nd = ndb
+    if (ndb != nda):
+        if (ndb > nda):
+            as_ = (1,)*(ndb-nda) + as_
+        else:
+            bs = (1,)*(nda-ndb) + bs
+            nd = nda
+    result = outer(a,b).reshape(as_+bs)
+    axis = nd-1
+    for _ in xrange(nd):
+        result = concatenate(result, axis=axis)
     if wrapper is not None:
         result = wrapper(result)
     return result
+
+
+def tile(A, reps):
+    """Repeat an array the number of times given in the integer tuple, reps.
+
+    If reps has length d, the result will have dimension of max(d, A.ndim).
+    If reps is scalar it is treated as a 1-tuple.
+
+    If A.ndim < d, A is promoted to be d-dimensional by prepending new axes.
+    So a shape (3,) array is promoted to (1,3) for 2-D replication,
+    or shape (1,1,3) for 3-D replication.
+    If this is not the desired behavior, promote A to d-dimensions manually
+    before calling this function.
+
+    If d < A.ndim, tup is promoted to A.ndim by pre-pending 1's to it.  Thus
+    for an A.shape of (2,3,4,5), a tup of (2,2) is treated as (1,1,2,2)
+
+
+    Examples:
+    >>> a = array([0,1,2])
+    >>> tile(a,2)
+    array([0, 1, 2, 0, 1, 2])
+    >>> tile(a,(1,2))
+    array([[0, 1, 2, 0, 1, 2]])
+    >>> tile(a,(2,2))
+    array([[0, 1, 2, 0, 1, 2],
+           [0, 1, 2, 0, 1, 2]])
+    >>> tile(a,(2,1,2))
+    array([[[0, 1, 2, 0, 1, 2]],
+    <BLANKLINE>
+           [[0, 1, 2, 0, 1, 2]]])
+
+    See Also:
+       repeat
+    """
+    try:
+        tup = tuple(reps)
+    except TypeError:
+        tup = (reps,)
+    d = len(tup)
+    c = _nx.array(A,copy=False,subok=True,ndmin=d)
+    shape = list(c.shape)
+    n = max(c.size,1)
+    if (d < c.ndim):
+        tup = (1,)*(c.ndim-d) + tup
+    for i, nrep in enumerate(tup):
+        if nrep!=1:
+            c = c.reshape(-1,n).repeat(nrep,0)
+        dim_in = shape[i]
+        dim_out = dim_in*nrep
+        shape[i] = dim_out
+        n /= max(dim_in,1)
+    return c.reshape(shape)
