@@ -1,11 +1,11 @@
 
-from os.path import join
-
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
     from numpy.distutils.system_info import get_info
     config = Configuration('linalg',parent_package,top_path)
 
+    config.add_data_dir('tests')
+
     # Configure lapack_lite
     lapack_info = get_info('lapack_opt',0)
     def get_lapack_lite_sources(ext, build_dir):
@@ -28,4 +28,4 @@ def configuration(parent_package='',top_path=None):
 
 if __name__ == '__main__':
     from numpy.distutils.core import setup
-    setup(**configuration(top_path='').todict())
+    setup(configuration=configuration)
