@@ -1,4 +1,3 @@
-
 import os
 import re
 import sys
@@ -7,17 +6,18 @@ import glob
 import types
 import unittest
 import traceback
+import warnings
 
 __all__ = ['set_package_path', 'set_local_path', 'restore_path',
            'IgnoreException', 'NumpyTestCase', 'NumpyTest',
            'ScipyTestCase', 'ScipyTest', # for backward compatibility
-           'importall'
+           'importall',
            ]
 
 DEBUG=0
+from numpy.testing.utils import jiffies
+from numpy.distutils.exec_command import splitcmdline
 get_frame = sys._getframe
-from utils import jiffies
-
 
 class IgnoreException(Exception):
     "Ignoring this exception due to disabled feature"
@@ -69,7 +69,7 @@ def set_local_path(reldir='', level=1):
         testfile = sys.argv[0]
     else:
         testfile = f.f_locals['__file__']
-    local_path = os.path.join(os.path.dirname(os.path.abspath(testfile)),reldir)
+    local_path = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(testfile)),reldir))
     if DEBUG:
         print 'Inserting %r to sys.path' % (local_path)
     sys.path.insert(0,local_path)
@@ -139,7 +139,7 @@ class NumpyTestCase (unittest.TestCase):
         result.stream = _dummy_stream(save_stream)
         unittest.TestCase.__call__(self, result)
         if nof_errors != len(result.errors):
-            test, errstr = result.errors[-1]
+            test, errstr = result.errors[-1][:2]
             if isinstance(errstr, tuple):
                 errstr = str(errstr[0])
             elif isinstance(errstr, str):
@@ -150,10 +150,12 @@ class NumpyTestCase (unittest.TestCase):
             l = len(result.stream.data)
             if errstr.startswith('IgnoreException:'):
                 if l==1:
-                    assert result.stream.data[-1]=='E',`result.stream.data`
+                    assert result.stream.data[-1]=='E', \
+                            repr(result.stream.data)
                     result.stream.data[-1] = 'i'
                 else:
-                    assert result.stream.data[-1]=='ERROR\n',`result.stream.data`
+                    assert result.stream.data[-1]=='ERROR\n', \
+                            repr(result.stream.data)
                     result.stream.data[-1] = 'ignoring\n'
                 del result.errors[-1]
         map(save_stream.write, result.stream.data)
@@ -191,7 +193,12 @@ class NumpyTestCase (unittest.TestCase):
                 runner.run(test)
         return
 
-ScipyTestCase = NumpyTestCase
+class ScipyTestCase(NumpyTestCase):
+    def __init__(self, package=None):
+        warnings.warn("ScipyTestCase is now called NumpyTestCase; please update your code",
+                         DeprecationWarning, stacklevel=2)
+        NumpyTestCase.__init__(self, package)
+
 
 def _get_all_method_names(cls):
     names = dir(cls)
@@ -204,7 +211,7 @@ def _get_all_method_names(cls):
 
 
 # for debug build--check for memory leaks during the test.
-class _SciPyTextTestResult(unittest._TextTestResult):
+class _NumPyTextTestResult(unittest._TextTestResult):
     def startTest(self, test):
         unittest._TextTestResult.startTest(self, test)
         if self.showAll:
@@ -221,37 +228,36 @@ class _SciPyTextTestResult(unittest._TextTestResult):
                               sys.gettotalrefcount()))
         return
 
-class SciPyTextTestRunner(unittest.TextTestRunner):
+class NumPyTextTestRunner(unittest.TextTestRunner):
     def _makeResult(self):
-        return _SciPyTextTestResult(self.stream, self.descriptions, self.verbosity)
+        return _NumPyTextTestResult(self.stream, self.descriptions, self.verbosity)
 
 
 class NumpyTest:
     """ Numpy tests site manager.
 
-    Usage:
-      >>> NumpyTest(<package>).test(level=1,verbosity=1)
+    Usage: NumpyTest(<package>).test(level=1,verbosity=1)
 
     <package> is package name or its module object.
 
-    Package is supposed to contain a directory tests/
-    with test_*.py files where * refers to the names of submodules.
-    See .rename() method to redefine name mapping between test_*.py files
-    and names of submodules. Pattern test_*.py can be overwritten by
-    redefining .get_testfile() method.
+    Package is supposed to contain a directory tests/ with test_*.py
+    files where * refers to the names of submodules.  See .rename()
+    method to redefine name mapping between test_*.py files and names of
+    submodules. Pattern test_*.py can be overwritten by redefining
+    .get_testfile() method.
 
-    test_*.py files are supposed to define a classes, derived
-    from NumpyTestCase or unittest.TestCase, with methods having
-    names starting with test or bench or check. The names of TestCase
-    classes must have a prefix test. This can be overwritten by
-    redefining .check_testcase_name() method.
+    test_*.py files are supposed to define a classes, derived from
+    NumpyTestCase or unittest.TestCase, with methods having names
+    starting with test or bench or check. The names of TestCase classes
+    must have a prefix test. This can be overwritten by redefining
+    .check_testcase_name() method.
 
     And that is it! No need to implement test or test_suite functions
     in each .py file.
 
-    Also old styled test_suite(level=1) hooks are supported.
+    Old-style test_suite(level=1) hooks are also supported.
     """
-    _check_testcase_name = re.compile(r'test.*').match
+    _check_testcase_name = re.compile(r'test.*|Test.*').match
     def check_testcase_name(self, name):
         """ Return True if name matches TestCase class.
         """
@@ -288,9 +294,14 @@ class NumpyTest:
         self._rename_map = {}
 
     def rename(self, **kws):
-        """ Apply renaming submodule test file test_<name>.py to test_<newname>.py.
-        Usage: self.rename(name='newname') before calling self.test() method.
-        If 'newname' is None, then no tests will be executed for a given module.
+        """Apply renaming submodule test file test_<name>.py to
+        test_<newname>.py.
+
+        Usage: self.rename(name='newname') before calling the
+        self.test() method.
+
+        If 'newname' is None, then no tests will be executed for a given
+        module.
         """
         for k,v in kws.items():
             self._rename_map[k] = v
@@ -300,7 +311,7 @@ class NumpyTest:
         filename = module.__file__[-30:]
         if filename!=module.__file__:
             filename = '...'+filename
-        return '<module %s from %s>' % (`module.__name__`, `filename`)
+        return '<module %r from %r>' % (module.__name__, filename)
 
     def _get_method_names(self,clsobj,level):
         names = []
@@ -333,7 +344,7 @@ class NumpyTest:
         short_module_name = self._rename_map.get(short_module_name,short_module_name)
         return short_module_name
 
-    def _get_module_tests(self,module,level,verbosity):
+    def _get_module_tests(self, module, level, verbosity):
         mstr = self._module_str
 
         short_module_name = self._get_short_module_name(module)
@@ -390,9 +401,8 @@ class NumpyTest:
 
     def _get_suite_list(self, test_module, level, module_name='__main__',
                         verbosity=1):
-        mstr = self._module_str
         suite_list = []
-        if hasattr(test_module,'test_suite'):
+        if hasattr(test_module, 'test_suite'):
             suite_list.extend(test_module.test_suite(level)._tests)
         for name in dir(test_module):
             obj = getattr(test_module, name)
@@ -404,12 +414,118 @@ class NumpyTest:
                 suite = obj(mthname)
                 if getattr(suite,'isrunnable',lambda mthname:1)(mthname):
                     suite_list.append(suite)
+        matched_suite_list = [suite for suite in suite_list \
+                              if self.testcase_match(suite.id()\
+                                                     .replace('__main__.',''))]
         if verbosity>=0:
-            self.info('  Found %s tests for %s' % (len(suite_list),module_name))
-        return suite_list
+            self.info('  Found %s/%s tests for %s' \
+                      % (len(matched_suite_list), len(suite_list), module_name))
+        return matched_suite_list
 
-    def test(self,level=1,verbosity=1):
-        """ Run Numpy module test suite with level and verbosity.
+    def _test_suite_from_modules(self, this_package, level, verbosity):
+        package_name = this_package.__name__
+        modules = []
+        for name, module in sys.modules.items():
+            if not name.startswith(package_name) or module is None:
+                continue
+            if not hasattr(module,'__file__'):
+                continue
+            if os.path.basename(os.path.dirname(module.__file__))=='tests':
+                continue
+            modules.append((name, module))
+
+        modules.sort()
+        modules = [m[1] for m in modules]
+
+        self.test_files = []
+        suites = []
+        for module in modules:
+            suites.extend(self._get_module_tests(module, abs(level), verbosity))
+
+        suites.extend(self._get_suite_list(sys.modules[package_name],
+                                           abs(level), verbosity=verbosity))
+        return unittest.TestSuite(suites)
+
+    def _test_suite_from_all_tests(self, this_package, level, verbosity):
+        importall(this_package)
+        package_name = this_package.__name__
+
+        # Find all tests/ directories under the package
+        test_dirs_names = {}
+        for name, module in sys.modules.items():
+            if not name.startswith(package_name) or module is None:
+                continue
+            if not hasattr(module, '__file__'):
+                continue
+            d = os.path.dirname(module.__file__)
+            if os.path.basename(d)=='tests':
+                continue
+            d = os.path.join(d, 'tests')
+            if not os.path.isdir(d):
+                continue
+            if test_dirs_names.has_key(d): continue
+            test_dir_module = '.'.join(name.split('.')[:-1]+['tests'])
+            test_dirs_names[d] = test_dir_module
+
+        test_dirs = test_dirs_names.keys()
+        test_dirs.sort()
+
+        # For each file in each tests/ directory with a test case in it,
+        # import the file, and add the test cases to our list
+        suite_list = []
+        testcase_match = re.compile(r'\s*class\s+\w+\s*\(.*TestCase').match
+        for test_dir in test_dirs:
+            test_dir_module = test_dirs_names[test_dir]
+
+            if not sys.modules.has_key(test_dir_module):
+                sys.modules[test_dir_module] = imp.new_module(test_dir_module)
+
+            for fn in os.listdir(test_dir):
+                base, ext = os.path.splitext(fn)
+                if ext != '.py':
+                    continue
+                f = os.path.join(test_dir, fn)
+
+                # check that file contains TestCase class definitions:
+                fid = open(f, 'r')
+                skip = True
+                for line in fid:
+                    if testcase_match(line):
+                        skip = False
+                        break
+                fid.close()
+                if skip:
+                    continue
+
+                # import the test file
+                n = test_dir_module + '.' + base
+                # in case test files import local modules
+                sys.path.insert(0, test_dir)
+                fo = None
+                try:
+                    try:
+                        fo = open(f)
+                        test_module = imp.load_module(n, fo, f,
+                                                      ('.py', 'U', 1))
+                    except Exception, msg:
+                        print 'Failed importing %s: %s' % (f,msg)
+                        continue
+                finally:
+                    if fo:
+                        fo.close()
+                    del sys.path[0]
+
+                suites = self._get_suite_list(test_module, level,
+                                              module_name=n,
+                                              verbosity=verbosity)
+                suite_list.extend(suites)
+
+        all_tests = unittest.TestSuite(suite_list)
+        return all_tests
+
+    def test(self, level=1, verbosity=1, all=False, sys_argv=[],
+             testcase_pattern='.*'):
+        """Run Numpy module test suite with level and verbosity.
 
         level:
           None           --- do nothing, return None
@@ -417,17 +533,28 @@ class NumpyTest:
                              don't run them, return TestSuite-list
           > 0            --- scan for tests of level, run them,
                              return TestRunner
+          > 10           --- run all tests (same as specifying all=True).
+                             (backward compatibility).
 
         verbosity:
           >= 0           --- show information messages
           > 1            --- show warnings on missing tests
 
-        It is assumed that package tests suite follows the following
-        convention: for each package module, there exists file
-        <packagepath>/tests/test_<modulename>.py that defines
+        all:
+          True            --- run all test files (like self.testall())
+          False (default) --- only run test files associated with a module
+
+        sys_argv          --- replacement of sys.argv[1:] during running
+                              tests.
+
+        testcase_pattern  --- run only tests that match given pattern.
+
+        It is assumed (when all=False) that package tests suite follows
+        the following convention: for each package module, there exists
+        file <packagepath>/tests/test_<modulename>.py that defines
         TestCase classes (with names having prefix 'test_') with methods
-        (with names having prefixes 'check_' or 'bench_'); each of
-        these methods are called when running unit tests.
+        (with names having prefixes 'check_' or 'bench_'); each of these
+        methods are called when running unit tests.
         """
         if level is None: # Do nothing.
             return
@@ -437,44 +564,34 @@ class NumpyTest:
         else:
             this_package = self.package
 
-        package_name = this_package.__name__
+        self.testcase_match = re.compile(testcase_pattern).match
 
-        modules = []
-        for name, module in sys.modules.items():
-            if package_name != name[:len(package_name)] \
-                   or module is None:
-                continue
-            if not hasattr(module,'__file__'):
-                continue
-            if os.path.basename(os.path.dirname(module.__file__))=='tests':
-                continue
-            modules.append(module)
+        if all:
+            all_tests = self._test_suite_from_all_tests(this_package,
+                                                        level, verbosity)
+        else:
+            all_tests = self._test_suite_from_modules(this_package,
+                                                      level, verbosity)
 
-        self.test_files = []
-        suites = []
-        for module in modules:
-            suites.extend(self._get_module_tests(module, abs(level), verbosity))
-
-        suites.extend(self._get_suite_list(sys.modules[package_name],
-                                           abs(level), verbosity=verbosity))
-
-        all_tests = unittest.TestSuite(suites)
-        if level<0:
+        if level < 0:
             return all_tests
 
         runner = unittest.TextTestRunner(verbosity=verbosity)
+        old_sys_argv = sys.argv[1:]
+        sys.argv[1:] = sys_argv
         # Use the builtin displayhook. If the tests are being run
         # under IPython (for instance), any doctest test suites will
         # fail otherwise.
         old_displayhook = sys.displayhook
         sys.displayhook = sys.__displayhook__
         try:
-            runner.run(all_tests)
+            r = runner.run(all_tests)
         finally:
             sys.displayhook = old_displayhook
-        return runner
+        sys.argv[1:] = old_sys_argv
+        return r
 
-    def testall(self,level=1,verbosity=1):
+    def testall(self, level=1,verbosity=1):
         """ Run Numpy module test suite with level and verbosity.
 
         level:
@@ -493,102 +610,7 @@ class NumpyTest:
         directory and no assumptions are made for naming the
         TestCase classes or their methods.
         """
-        if level is None: # Do nothing.
-            return
-
-        if isinstance(self.package, str):
-            exec 'import %s as this_package' % (self.package)
-        else:
-            this_package = self.package
-        package_name = this_package.__name__
-
-        importall(this_package)
-
-        test_dirs_names = {}
-        for name, module in sys.modules.items():
-            if package_name != name[:len(package_name)] \
-                   or module is None:
-                continue
-            if not hasattr(module,'__file__'):
-                continue
-            d = os.path.dirname(module.__file__)
-            if os.path.basename(d)=='tests':
-                continue
-            d = os.path.join(d, 'tests')
-            if not os.path.isdir(d):
-                continue
-            if test_dirs_names.has_key(d): continue
-            test_dir_module = '.'.join(name.split('.')[:-1]+['tests'])
-            test_dirs_names[d] = test_dir_module
-
-        test_dirs = test_dirs_names.keys()
-        test_dirs.sort()
-
-        suite_list = []
-        testcase_match = re.compile(r'\s*class\s+[_\w]+\s*\(.*TestCase').match
-        for test_dir in test_dirs:
-            test_dir_module = test_dirs_names[test_dir]
-
-            if not sys.modules.has_key(test_dir_module):
-                sys.modules[test_dir_module] = imp.new_module(test_dir_module)
-            
-            for fn in os.listdir(test_dir):
-                base, ext = os.path.splitext(fn)
-                if ext != '.py':
-                    continue
-                f = os.path.join(test_dir, fn)
-
-                # check that file contains TestCase class definitions:
-                fid = open(f)
-                skip = True
-                for line in fid.readlines():
-                    if testcase_match(line):
-                        skip = False
-                        break
-                fid.close()
-                if skip:
-                    continue
-
-                # import the test file
-                n = test_dir_module + '.' + base
-                sys.path.insert(0,test_dir) # in case test files import local modules
-                try:
-                    test_module = imp.load_module(n,
-                                                  open(f),
-                                                  f,
-                                                  ('.py', 'U', 1))
-                except Exception, msg:
-                    print 'Failed importing %s: %s' % (f,msg)
-                    del sys.path[0]
-                    continue
-                del sys.path[0]
-
-                for name in dir(test_module):
-                    obj = getattr(test_module, name)
-                    if type(obj) is not type(unittest.TestCase) \
-                           or not issubclass(obj, unittest.TestCase) \
-                           or not self.check_testcase_name(obj.__name__):
-                        continue
-                    for mthname in self._get_method_names(obj,level):
-                        suite = obj(mthname)
-                        if getattr(suite,'isrunnable',lambda mthname:1)(mthname):
-                            suite_list.append(suite)
-
-        all_tests = unittest.TestSuite(suite_list)
-        if level<0:
-            return all_tests
-       
-        runner = unittest.TextTestRunner(verbosity=verbosity)
-        # Use the builtin displayhook. If the tests are being run
-        # under IPython (for instance), any doctest test suites will
-        # fail otherwise.
-        old_displayhook = sys.displayhook
-        sys.displayhook = sys.__displayhook__
-        try:
-            runner.run(all_tests)
-        finally:
-            sys.displayhook = old_displayhook
-        return runner
+        return self.test(level=level, verbosity=verbosity, all=True)
 
     def run(self):
         """ Run Numpy module test suite with level and verbosity
@@ -599,7 +621,9 @@ class NumpyTest:
         except ImportError:
             self.warn('Failed to import optparse module, ignoring.')
             return self.test()
-        usage = r'usage: %prog [-v <verbosity>] [-l <level>]'
+        usage = r'usage: %prog [-v <verbosity>] [-l <level>]'\
+                r' [-s "<replacement of sys.argv[1:]>"]'\
+                r' [-t "<testcase pattern>"]'
         parser = OptionParser(usage)
         parser.add_option("-v", "--verbosity",
                           action="store",
@@ -611,9 +635,20 @@ class NumpyTest:
                           dest="level",
                           default=1,
                           type='int')
+        parser.add_option("-s", "--sys-argv",
+                          action="store",
+                          dest="sys_argv",
+                          default='',
+                          type='string')
+        parser.add_option("-t", "--testcase-pattern",
+                          action="store",
+                          dest="testcase_pattern",
+                          default=r'.*',
+                          type='string')
         (options, args) = parser.parse_args()
-        self.test(options.level,options.verbosity)
-        return
+        return self.test(options.level,options.verbosity,
+                         sys_argv=splitcmdline(options.sys_argv or ''),
+                         testcase_pattern=options.testcase_pattern)
 
     def warn(self, message):
         from numpy.distutils.misc_util import yellow_text
@@ -623,8 +658,13 @@ class NumpyTest:
         print>>sys.stdout, message
         sys.stdout.flush()
 
-ScipyTest = NumpyTest
-        
+class ScipyTest(NumpyTest):
+    def __init__(self, package=None):
+        warnings.warn("ScipyTest is now called NumpyTest; please update your code",
+                         DeprecationWarning, stacklevel=2)
+        NumpyTest.__init__(self, package)
+
+
 def importall(package):
     """
     Try recursively to import all subpackages under package.
